//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: XmlDocument.cpp,v 1.63 2004/12/03 21:25:07 gmf Exp $
//

#include "dbxml_config.h"
#include <dbxml/XmlPortability.hpp>
#include <dbxml/XmlDocument.hpp>
#include <dbxml/XmlResults.hpp>
#include <dbxml/XmlQueryContext.hpp>
#include <dbxml/XmlQueryExpression.hpp>
#include <dbxml/XmlMetaDataIterator.hpp>
#include <dbxml/XmlData.hpp>
#include "Document.hpp"
#include "QueryContext.hpp"
#include "UTF8.hpp"
#include "QueryExpression.hpp"
#include "Value.hpp"
#include "Results.hpp"

#include <xercesc/parsers/XercesDOMParser.hpp>
#if defined(XERCES_HAS_CPP_NAMESPACE)
XERCES_CPP_NAMESPACE_USE
#endif

#include <pathan/PathanImplementation.hpp>
#include <pathan/DynamicContext.hpp>
#include <pathan/internal/XPath2MemoryManagerImpl.hpp>

using namespace DbXml;

XmlDocument::XmlDocument(Document *document)
	: document_(document)
{
	document_->acquire();
}

XmlDocument::~XmlDocument()
{
	if (document_ != 0)
		document_->release();
}

XmlDocument::XmlDocument(const XmlDocument &o)
	: document_(o.document_)
{
	if (document_ != 0)
		document_->acquire();
}

XmlDocument &XmlDocument::operator=(const XmlDocument &o)
{
	if (this != &o && document_ != o.document_) {
		if (document_ != 0)
			document_->release();
		document_ = o.document_;
		if (document_ != 0)
			document_->acquire();
	}
	return *this;
}

void XmlDocument::setName(const std::string &name)
{
	document_->setName(name, /*modified*/true);
}

std::string XmlDocument::getName() const
{
	return document_->getName();
}

void XmlDocument::setContent(const std::string &content)
{
	DbtOut *dbtContent = new DbtOut((void*)content.c_str(), content.length());
	document_->setContentAsDbt(&dbtContent);
}

void XmlDocument::setContent(const XmlData &content)
{
	DbtOut *dbtContent = new DbtOut(content.getDbt());
	document_->setContentAsDbt(&dbtContent);
}

void XmlDocument::setContentAsXmlInputStream(XmlInputStream *adopted_str)
{
	document_->setContentAsInputStream(&adopted_str);
}

XmlInputStream *XmlDocument::getContentAsXmlInputStream() const
{
	return document_->getContentAsInputStream();
}

XmlData XmlDocument::getContent() const
{
	const Dbt *dbt = document_->getContentAsDbt();
	if (dbt != 0 && dbt->get_size() > 0) {
		return XmlData(*dbt);
	}
	return XmlData();
}

std::string &XmlDocument::getContent(std::string &s) const
{
	const Dbt *dbt = document_->getContentAsDbt();
	if (dbt != 0 && dbt->get_size() > 0) {
		s.assign((const char *)dbt->get_data(), dbt->get_size());
	} else {
		s.erase();
	}
	return s;
}

void XmlDocument::fetchAllData()
{
	document_->setEagerContentAndMetaData();
}

void XmlDocument::setMetaData(const std::string &uri, const std::string &name, const XmlValue &value)
{
	Name n(uri.c_str(), name.c_str());
	document_->setMetaData(n, value, /*modified*/true);
}

bool XmlDocument::getMetaData(const std::string &uri, const std::string &name, XmlValue &value) const
{
	Name n(uri.c_str(), name.c_str());
	return document_->getMetaData(n, value);
}

void XmlDocument::setMetaData(const std::string &uri, const std::string &name, const XmlData &value)
{
	Name n(uri.c_str(), name.c_str());
	document_->setMetaData(n, static_cast<XmlValue::Type>(Value::BINARY), value.getDbt(), /*modified*/true);
}

bool XmlDocument::getMetaData(const std::string &uri, const std::string &name, XmlData &value) const
{
	Name n(uri.c_str(), name.c_str());
	return document_->getMetaData(n, value.getDbt());
}

void XmlDocument::removeMetaData(const std::string &uri, const std::string &name)
{
	Name n(uri.c_str(), name.c_str());
	document_->removeMetaData(n);
}

XmlMetaDataIterator XmlDocument::getMetaDataIterator() const
{
	return new MetaDataIterator(document_);
}

void XmlDocument::setContentAsDOM(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document)
{
	document_->setContentAsDOM(document);
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *XmlDocument::getContentAsDOM() const
{
	return document_->getContentAsDOM();
}
