//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: XmlQueryContext.cpp,v 1.26 2004/10/18 22:52:09 gmf Exp $
//

#include "dbxml_config.h"
#include "dbxml/XmlPortability.hpp"
#include "dbxml/XmlQueryContext.hpp"
#include "dbxml/XmlManager.hpp"
#include "QueryContext.hpp"

using namespace DbXml;
using namespace std;

XmlQueryContext::XmlQueryContext(QueryContext *context)
  : queryContext_(context)
{
  queryContext_->acquire();
}

XmlQueryContext::~XmlQueryContext()
{
	queryContext_->release();
}

XmlQueryContext::XmlQueryContext(const XmlQueryContext &o)
	: queryContext_(o.queryContext_)
{
	if (queryContext_ != 0)
		queryContext_->acquire();
}

XmlQueryContext &XmlQueryContext::operator=(const XmlQueryContext &o)
{
	if (this != &o && queryContext_ != o.queryContext_) {
		if (queryContext_ != 0)
			queryContext_->release();
		queryContext_ = o.queryContext_;
		if (queryContext_ != 0)
			queryContext_->acquire();
	}
	return *this;
}

void XmlQueryContext::setVariableValue(const string &name, const XmlValue &value)
{
	queryContext_->setVariableValue(name, value);
}

bool XmlQueryContext::getVariableValue(const string &name, XmlValue &value) const
{
	return queryContext_->getVariableValue(name, value);
}

void XmlQueryContext::setNamespace(const string &prefix, const string &uri )
{
	queryContext_->setNamespace(prefix, uri);
}

string XmlQueryContext::getNamespace(const string &prefix )
{
	return queryContext_->getNamespace(prefix);
}

void XmlQueryContext::removeNamespace(const string &prefix )
{
	queryContext_->removeNamespace(prefix);
}

void XmlQueryContext::clearNamespaces()
{
	queryContext_->clearNamespaces();
}

void XmlQueryContext::setBaseURI(const string &baseURI)
{
  queryContext_->setBaseURI(baseURI);
}

string XmlQueryContext::getBaseURI() const
{
  return queryContext_->getBaseURI();
}

void XmlQueryContext::setReturnType(ReturnType rt)
{
	queryContext_->setReturnType(rt);
}

XmlQueryContext::ReturnType XmlQueryContext::getReturnType() const
{
	return queryContext_->getReturnType();
}

void XmlQueryContext::setEvaluationType(EvaluationType et)
{
	queryContext_->setEvaluationType(et);
}

XmlQueryContext::EvaluationType XmlQueryContext::getEvaluationType() const
{
	return queryContext_->getEvaluationType();
}
