//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: CreateContainerCommand.cpp,v 1.7 2004/11/12 15:27:55 jsnelson Exp $
//

#include "CreateContainerCommand.hpp"

using namespace DbXml;
using namespace std;

string CreateContainerCommand::getCommandName() const
{
  return "create";
}

string CreateContainerCommand::getBriefHelp() const
{
  return "Creates a container, and sets the default container to it";
}

string CreateContainerCommand::getMoreHelp() const
{
  return
	  string("Takes the name of the container as an argument, as well as an\n") +
	  string("optional argument of \"n\" for a node storage container, or \"in\"\n") +
	  string("for a node storage container with DBXML_INDEX_NODES specified.\n") +
	  string("This command uses the XmlManager::createContainer() method.\n");
}

void CreateContainerCommand::execute(Args &args, Environment &env)
{
  if(args.size() < 2 || args.size() > 3) {
    throw CommandException("Wrong number of arguments");
  }
  env.deleteContainer();

  XmlContainer::ContainerType cType = XmlContainer::WholedocContainer;
  u_int32_t flags = 0;
  if(args.size() == 3) {
	  if(args[2] == "n") {
		  cType = XmlContainer::NodeContainer;
		  if(env.verbose()) cout << "Creating node storage container" << endl;
	  }
	  else if(args[2] == "in") {
		  cType = XmlContainer::NodeContainer;
		  flags = DBXML_INDEX_NODES;
		  if(env.verbose()) cout << "Creating node storage container with nodes indexed" << endl;
	  }
	  else {
		  throw CommandException("Unknown container type");
	  }
  }
  else if(env.verbose()) cout << "Creating document storage container" << endl;

  if(env.transactions()) flags |= DBXML_TRANSACTIONAL;
  env.container() = new XmlContainer(env.db().createContainer(args[1], flags, cType));
}

