# See the file LICENSE for redistribution information.
#
# Copyright (c) 2000-2004
#	Sleepycat Software.  All rights reserved.
#
# $Id: batch_load.tcl,v 1.3 2004/10/14 16:19:50 awood Exp $

# Batch execution of the container loading (benchmarking)
# Use comments / adjust lists to configure what is done!
#
# Invocation
#   tclsh batch_load ?
#   tclsh batch_load <dbenv> <data>
#
# Arguments
#   <dbenv>  - path to DB XML environment directory
#   <data>   - path to directory that contains XBench generated XML data
#   <path>   - path to directory that contains utility executables
#

proc usage { name } {
    puts "Usage: $name <dbenv> <data> <path>"
    puts "\t<dbenv> - path to DB XML environment"
    puts "\t<data> - path to directory containing XBench data"
    puts "\t<path> - path to directory containing DB XML utility executables"
}

# help option
if { $argc == 1 && [lindex $argv 0] == "?"} {
    usage $argv0
    exit
}

set err 1

# verify arguments
if { $argc != 3 } {
    puts "Incorrect number of arguments: $argc"
    usage $argv0
    exit $err
}

set dbenv [lindex $argv 0]
set data_dir [lindex $argv 1]
set exe_path [lindex $argv 2]

# load the procedures 
set xbench_tcl [file join [file dirname $argv0] xbench.tcl]
source $xbench_tcl

# verify the XBench data directories
set groups [list "DC-MD" "DC-SD" "TC-MD" "TC-SD"]
foreach group $groups {
    set dir [file join $data_dir $group]
    if {[file exists $dir] != 1} {
        puts "Cannot find the required XBench data sub-directory \"$dir\""
        exit $err
    }
}

# verify the path
if {[file exists [file join $exe_path "load_container"] ] != 1} {
    puts "Cannot find load_container in \"$exe_path\""
    exit $err
}

################################################################
# Container loading
#  modify the lists to control what is loaded!

puts "Loading containers..."

#set sizes [list "miniscule" "tiny" "small" "medium" "normal" "large"]
set sizes [list "miniscule"]
set types [list "node" "wholedoc"]
foreach size $sizes {
    puts "\tSize \"$size\""
    foreach type $types {
        puts "\t\tContainer type \"$type\""
        set err [load_containers $data_dir $dbenv $exe_path $size $type]
        if { $err != 0 } {
            puts "Error while loading container"
            exit $err
        }
    }
}

set err 0
exit $err
