#!/usr/bin/perl --
use strict;
use XML::XPath;
use Cwd;

my $pwd = cwd();
my $configFile = "benchmark.xml";
my $dbscript = "xdbgen.pl";
my $xbenchDir = ".";
my $containerCreator = "LoadContainer";
my $queryRunner = "QueryRunner";
my $createDatabase = 0;
my $includeResults = 0;
my $outputFile = "";

my $index = 1;
foreach my $arg (@ARGV) {
    if ($arg eq "-c") {
	$configFile = @ARGV[$index];
	if($configFile eq "") {
	    print "You must specify a config file with the -c option";
	    exit(-1);
	}
    }
    elsif ($arg eq "-x") {
	$xbenchDir = @ARGV[$index];
	if($xbenchDir eq "") {
	    print "You must specify an xbench dir with the -x option";
	    exit(-1);
	}
    }
    elsif ($arg eq "-l") {
	$containerCreator = @ARGV[$index];
	if($xbenchDir eq "") {
	    print "You must specify a load container path with the -l option";
	    exit(-1);
	}
    }
    elsif ($arg eq "-q") {
	$queryRunner = @ARGV[$index];
	if($queryRunner eq "") {
	    print "You must specify a query runner path with the -q option";
	    exit(-1);
	}
    }
    elsif ($arg eq "-i") {
	$createDatabase = 1;
    }
    elsif ($arg eq "-o") {
	$includeResults = 1;
    }
    elsif ($arg eq "-f") {
	$outputFile = @ARGV[$index];
	if($outputFile eq "") {
	    print "You must specify an output file with the -f option";
	    exit(-1);
	}
    }

    elsif($arg eq "-h") {
	print "This script generates the xml files and loads them into containers specified in the configuration file\n";
	print "Options:\n";
	print "-c config file (default benchmark.xml)\n";
	print "-x xbench directory (default .)\n";
	print "-o include the results of the queries in the XML output\n";
	print "-i create the XML files and containers specified in the config file\n";
	print "-f the file to write the XML output to (default to std out)\n";
	print "-l path and executable for LoadContainer (default LoadContainer)\n"; 
	print "-q path and executable for QueryRunner (default QueryRunner)\n"; 
	exit(1);
    }

    $index++;
}

if($createDatabase) {

    if(!-e $containerCreator) {
	die "Load container executable '$containerCreator' does not exist";
    }

#configure the database
    my $xp = XML::XPath->new(filename => $configFile) or die("Could not find config file $configFile");
    my $dbName = $xp->find('/xqueryrunner/@database');
    foreach my $container ($xp->find('/xqueryrunner/containerDescriptor/container')->get_nodelist){

	my $class = $container->find('class')->string_value;
	my $size = $container->find('size')->string_value;
	my $name = $container->find('name')->string_value;

	if($class eq "" ) {
	    print "Must specify class element (1, 2, 3 or 4)\n";
	    exit(-1);
	}

	if($size eq "" ) {
	    print "Must specify size element (1, 2, 3 or 4)\n";
	    exit(-1);
	}

	if($name eq "" ) {
	    print "Must specify name element - the name of the container\n";
	    exit(-1);
	}

	chdir($xbenchDir) or die "XBench directory does not exist\n";

	open(PIPE,"| perl $dbscript") || die("Cannot not xbench script $dbscript");
	print PIPE "$class\n";
	print PIPE "$size\n";
	close(PIPE);
	chdir($pwd);
	
	opendir(DIR,"$xbenchDir/output/") || die "opendir $xbenchDir/output/ failed";
	my @fileList = grep(!/^\.\.?$/,readdir(DIR));
	closedir(DIR);
	
	foreach my $currentFile(@fileList) {
	    $currentFile = "\"$xbenchDir/output/$currentFile\"";
	}
	
	mkdir($dbName);

	print "$containerCreator -d $dbName -c $name @fileList\n";
	system("$containerCreator -d $dbName -c $name @fileList");

	print "loaded files into container\n";
    }
}



if(!-e $queryRunner) {
    die "Query Runner executable '$queryRunner' does not exist";
}

my $tmpString;

if($includeResults) {
    $tmpString .= "-o ";
}

if($outputFile ne "") {
    $tmpString .= "-f $outputFile";
}

#run the actual tests
print("$queryRunner -c $configFile $tmpString");
system("$queryRunner -c $configFile $tmpString") or die "Unable to run benchmark tests";
